@@ -22,7 +22,7 @@
 #ifndef PHP_SUHOSIN_H
 #define PHP_SUHOSIN_H
 
-#define SUHOSIN_EXT_VERSION  "0.9.35"
+#define SUHOSIN_EXT_VERSION  "0.9.36"
 
 /*#define SUHOSIN_DEBUG*/
 #define SUHOSIN_LOG "/tmp/suhosin_log.txt"
@@ -192,6 +192,7 @@ ZEND_BEGIN_MODULE_GLOBALS(suhosin)
 
 /*	session */
 	void	*s_module;
+	void	*s_original_mod;
 	int 	(*old_s_read)(void **mod_data, const char *key, char **val, int *vallen TSRMLS_DC);
 	int	(*old_s_write)(void **mod_data, const char *key, const char *val, const int vallen TSRMLS_DC);
 	int	(*old_s_destroy)(void **mod_data, const char *key TSRMLS_DC);
@@ -239,6 +240,9 @@ ZEND_BEGIN_MODULE_GLOBALS(suhosin)
 	php_uint32   *mt_next;
 	int          mt_left;
 
+	char         *seedingkey;
+	zend_bool    reseed_every_request;
+
 	zend_bool r_is_seeded; 
 	zend_bool mt_is_seeded;
 
